//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     check.cxx
// Purpose:    Runtime Assertion Check
// Last Change:23.03.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 1997-2001 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"

#include <stdlib.h>
#include <iostream.h>
#include <iomanip.h>

#ifdef   VER_SIGNAL
#include <signal.h>
#endif

#include "out.h"
#include "check.h"

//=============================================================================
// Forward Declarations for Local Functions:
//=============================================================================

#ifdef VER_SIGNAL
#ifdef VER_DEBUG
static	void	check_sig(int sig);
#endif
#endif

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// check_init: Catch "segmentation fault", print error message to stdout.
//-----------------------------------------------------------------------------

#ifdef VER_DEBUG
void	check_init(void)
{
#ifdef VER_SIGNAL
	signal(SIGSEGV, &check_sig);
	signal(SIGBUS, &check_sig);
	signal(SIGFPE, &check_sig);
#endif
}
#endif

//-----------------------------------------------------------------------------
// check_failed: Print error message and abort program.
//-----------------------------------------------------------------------------

#ifdef VER_DEBUG
void	check_failed(str_t file, int line, str_t msg)
{
	// Print error message:
	out_c::assertion_failed(file, line, msg);

	// Perhaps someday I will automatically send me a mail ...

	// Terminate program:
#ifdef VER_UNIX
	abort();
#else
	exit(3);
#endif
}
#endif

//=============================================================================
// Local Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// check_sig: Signal handler.
//-----------------------------------------------------------------------------

#ifdef VER_SIGNAL
#ifdef VER_DEBUG
void check_sig(int sig)
{
	str_t sig_name;

	// Determine signal name:
	if(sig == SIGSEGV)
		sig_name = "Segmentation Fault";
	else if(sig == SIGBUS)
		sig_name = "Bus Error";
	else if(sig == SIGFPE)
		sig_name = "Arithmetic Error";
	else
		sig_name = "Unexpected Signal";

	// Print error message:
	out_c::exception(sig, sig_name);

	// Perhaps someday I will automatically send me a mail ...

	// Terminate program:
	exit(3);
}
#endif
#endif

